static char version[] = "zw2hz 2.0 (July 7, 1992)";

/*
  Copyright (C) 1989, 1992      Fung F. Lee

  zw2hz 2.0: do a straightforward conversion from a zW file into a HZ file

  This version was an update of version 1.1, because the specification of
  zW had been changed by the original authors.

  Since the set of all zW files is a proper subset of the set of all
  HZ (HGB) files, it is always possible to do perfect translation from
  zW to HZ (HGB); but not vice versa. 
  * HGB - High-bit-set GB, as used in CCDOS, Macintosh System 6.0.x and later.

  As for error handling, I took the lazy approach. For example, if the
  original zW file contains invalid GB codes, they will also show up in
  the output HZ file, and can be detected by "hz2gb -v".


  This program is free for general distribution.  

  This program runs on UNIX. You are welcome to port it to other operating
  systems.

*/


#include <stdio.h>

#define TRUE		1
#define FALSE		0

void usage(), filter();

void usage()
{
    fprintf(stderr, "This is %s. Copyright 1989-1992 Fung F. Lee\n\n", version);
    fprintf(stderr, "usage: zw2hz < foo.zw > foo.hz\n");
    exit(1); 
}


main(argc, argv)
int argc;
char *argv[];
{
    if (argc > 1) usage();
    filter(stdin, stdout);
}


/* As we do not want to impose any limit of line length (such as 80 characters
   per line), we parse the input stream on a character by character basis,
   because in the worst case, a line can be as long as a file.
   Although in practice the line length (with or without soft CR marker at
   its end) is likely to be about 80 characters or so, I am not sure what 
   the maximum line length is enforced by the zW standard, nor do I think
   it is a necessary assumption for proper decoding.
 */

void filter(fin, fout)
FILE *fin, *fout;
{
int c1, c2;
int ASCIImode = TRUE;
int lineStart = TRUE;
    
    while ((c1 = fgetc(fin)) != EOF)
    {
	if (ASCIImode)
	{
	    if (c1 == '\n')
	    {
		fputc('\n', fout);
		lineStart = TRUE;
	    }
	    else if (lineStart && c1 == 'z')
	    {
		c2 = fgetc(fin);
		if (c2 == EOF) {fputc(c1, fout); break;}
		if (c2 == 'W')
		{
		    fprintf(fout, "~{");
		    ASCIImode = FALSE;
		}
		else
		{
		    fputc(c1, fout);
		    fputc(c2, fout);
		}
		lineStart = FALSE;
	    }
	    else
	    {
		fputc(c1, fout);
		lineStart = FALSE;
	    }
	}
	else /* GBmode */
	{
	    c2 = fgetc(fin);
	    if (c1 == '\n')
	    {
		ungetc(c2, fin);
		fprintf(fout, "~}~\n"); /* soft CR - with line continuation */
		lineStart = TRUE;
		ASCIImode = TRUE;
	    }
	    else if (c2 == EOF) 
	    {
		fputc(c1, fout);
		break;
	    }
	    else if (c1 == '#' && c2 == '\n')
	    {
		fprintf(fout, "~}\n"); /* hard CR */
		lineStart = TRUE;
		ASCIImode = TRUE;
	    }
	    else if (c2 == '\n')  /* This may be an invalid zW sequence, ... */
	    {			  /* anyway, for robustness, I choose ... */
		/* eat c1 */	  /* c1 may be ' ' or something else */
		fprintf(fout, "~}\n"); /* hard CR */
		lineStart = TRUE;
		ASCIImode = TRUE;
	    }
	    else if (c1 == '#' && c2 == ' ')
	    {
		fprintf(fout, "~} ~{");  /* temporary escape and back */
	    }
	    else if (c1 == ' ')   /* 0x20?? is now for ASCII characters */
	    {
		fprintf(fout, "~}%c~{", c2);  /* temporary escape and back */
	    }
	    else  /* ASSUME they are GB codes, and fix them in program hz2gb */
	    {
		fputc(c1, fout); fputc(c2, fout);
	    }
	}
    }
}





